/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac             

   class      : ChangePasswdDlg
   date       : 5.12.97
   description: a dialog that prompts for a new password
   modified   : 2.2.98 - auto cancel mode for applet operation added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/


import java.awt.*; 
import java.awt.event.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class ChangePasswdDlg extends Dialog
  implements ActionListener,TextListener {

  // instance variable declarations
  private Frame parent;
  private boolean canceled   = false;
  private boolean autoCancel = false;
  private Button okButton;
  private Button cancelButton;
  private TextField oldPasswdText;
  private TextField newPasswdText;
  private TextField retypePasswdText;
  private String oldPasswd;
  private Component focusRequester = null;

  // inner classes
  class ChangePasswdDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setCanceled(true);
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowActivated(WindowEvent e) {
      if (focusRequester != null) {
        focusRequester.requestFocus();
        focusRequester = null;
      } // end if
    } // end WindowActivated

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setCanceled(true);
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end ChangePasswdDlgListener

  // costructors
  public ChangePasswdDlg(Frame parent, String oldPasswd) {
    super(parent);
    this.parent    = parent;
    this.oldPasswd = oldPasswd;
    init();
  } // end constructor ChangePasswdDlg

  public ChangePasswdDlg(Frame parent, boolean modal, String oldPasswd) {
    super(parent,modal);
    this.parent    = parent;
    this.oldPasswd = oldPasswd;
    init();
  } // end constructor ChangePasswdDlg

  public ChangePasswdDlg(Frame parent, String title, String oldPasswd) {
    super(parent,title);
    this.parent    = parent;
    this.oldPasswd = oldPasswd;
    init();
  } // end constructor ChangePasswdDlg

  public ChangePasswdDlg(Frame parent, String title,
                         boolean modal, String oldPasswd) {
    super(parent,title,modal);
    this.parent    = parent;
    this.oldPasswd = oldPasswd;
    init();
  } // end constructor ChangePasswdDlg

  // methods
  private void init() {
    addWindowListener(new ChangePasswdDlgListener());
    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();

    // create the widgets
    InsetsPanel pan = new InsetsPanel(new Insets(15,20,15,20));
    okButton = new Button("OK");
    okButton.setEnabled(false);
    okButton.addActionListener(this);
    cancelButton = new Button("Cancel");
    cancelButton.addActionListener(this);
    InsetsPanel buttonPanel = new InsetsPanel(new Insets(15,0,0,0));
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);

    oldPasswdText = new TextField("",15);
    oldPasswdText.setEchoChar('*');
    oldPasswdText.addTextListener(this);
    newPasswdText = new TextField("",15);
    newPasswdText.setEchoChar('*');
    newPasswdText.addTextListener(this);
    retypePasswdText = new TextField("",15);
    retypePasswdText.setEchoChar('*');
    retypePasswdText.addTextListener(this);
    Panel editPanel = new Panel();
    editPanel.setLayout(new GridLayout(3,2));
    editPanel.add(new Label("Old password:"));
    editPanel.add(oldPasswdText);
    editPanel.add(new Label("New password:"));
    editPanel.add(newPasswdText);
    editPanel.add(new Label("Retype new password:"));
    editPanel.add(retypePasswdText);

    pan.setLayout(gridbag); 

    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER; 
    constraints.gridheight = GridBagConstraints.RELATIVE; 
    gridbag.setConstraints(editPanel,constraints); 
    pan.add(editPanel); 

    constraints.ipadx      = 0; 
    constraints.ipady      = 0; 
    constraints.weightx    = 0.0; 
    constraints.weighty    = 0.0; 
    constraints.gridwidth  = 1; 
    constraints.gridheight = GridBagConstraints.REMAINDER; 

    gridbag.setConstraints(buttonPanel, constraints); 
    pan.add(buttonPanel); 

    add(pan);
    pack(); 
    setResizable(false);
    //setVisible(true);
  } // end init

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Cancel")) {
      canceled = true;
      setVisible(false);
      dispose();
    }
    else {
      String str = oldPasswdText.getText();
      if (!oldPasswd.equals(str)) {
        oldPasswdText.setText("");
        newPasswdText.setText("");
        retypePasswdText.setText("");
        focusRequester = oldPasswdText;
        new MsgBoxDlg(parent,getTitle(),isModal(),
          "The old password is wrong. Please type it again.",
          MsgBoxDlg.OK | MsgBoxDlg.ICONINFORMATION);
      }
      else if (!newPasswdText.getText().equals(retypePasswdText.getText())) {
        newPasswdText.setText("");
        retypePasswdText.setText("");
        focusRequester = newPasswdText;
        new MsgBoxDlg(parent,getTitle(),isModal(),
          "The retyped password doesn't match to the new password.\n"+
          "Please type it again.",
          MsgBoxDlg.OK | MsgBoxDlg.ICONINFORMATION);
      }
      else {
        setVisible(false);
        dispose();
      } // end if
    } // end if
  } // end actionPerformed

  public void textValueChanged(TextEvent e) {
    int oldLength = oldPasswdText.getText().length();
    int newLength = newPasswdText.getText().length();
    int retLength = retypePasswdText.getText().length();
    okButton.setEnabled(((oldLength >= 1) ||
                         (newLength >= 1)) &&
                        (retLength == newLength));
  } // end textValueChanged

  public boolean getCanceled() {
    return canceled;
  } // end getCanceled

  public void setCanceled(boolean canceled) {
    this.canceled = canceled;
  } // end getCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

  public String getPasswd() {
    return newPasswdText.getText();
  } // end getPasswd

  public void setPasswd(String passwd) {
    newPasswdText.setText(passwd);
    retypePasswdText.setText(passwd);
  } // end setPasswd

} // end ChangePasswdDlg

/* ******************************************************************
   end of file
*********************************************************************/
