/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : DBCanvas
   date       : 02.03.98
   description: a canvas that supports double buffering

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.image.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class DBCanvas extends Canvas {

  // instance variable declarations
  private Image offScreenBuffer;
  private boolean enabled = true;
  private boolean invalid = true;

  // constructors
  public DBCanvas() {
    super();
  } // end constructor DBCanvas

  public DBCanvas(boolean enabled) {
    super();
    this.enabled = enabled;
  } // end constructor DBCanvas

  // methods
  public void setBufferEnabled(boolean enabled) {
    if (enabled) {
      if (!this.enabled) {
        this.invalid = true;
        this.enabled = true;
      } // end if
    }
    else {
      if (this.enabled) {
        if (offScreenBuffer != null) {
          offScreenBuffer.flush();
          offScreenBuffer = null;
        } // end if
        this.enabled = false;
      } // end if
    } // end if
  } // end setBufferEnabled

  public boolean getBufferEnabled() {
    return enabled;
  } // end getBufferEnabled

  public void setBufferInvalid() {
    invalid = true;
  } // end setBufferInvalid

  public Graphics getBufferGraphics() {
    if (offScreenBuffer == null)
      return null;
    else
      return offScreenBuffer.getGraphics();
    // end if
  } // end getBufferGraphics

  public void bufferedPaint(Graphics g) {
    // this method should be overwritten be the class witch extends DBCanvas
  } // end bufferedPaint

  public void paint(Graphics g) {
    if (!enabled) {
      bufferedPaint(g);
      return;
    } // end if
    Graphics gr;
    // Will hold the graphics context from the offScreenBuffer.
    // We need to make sure we keep our offscreen buffer the same size
    // as the graphics context we're working with.
    if (offScreenBuffer==null ||
        (! (offScreenBuffer.getWidth(this) == this.getSize().width &&
            offScreenBuffer.getHeight(this) == this.getSize().height))) {
      offScreenBuffer = this.createImage(getSize().width,getSize().height);
    } // end if

    if (invalid) {
      // We need to use our buffer Image as a Graphics object:
      gr = offScreenBuffer.getGraphics();
      gr.setClip(0,0,this.getSize().width,this.getSize().height);
      bufferedPaint(gr); // Passes our off-screen buffer to our bufferedPaint
                         // method, which, unsuspecting, paints on it just as
                         // it would on the Graphics passed by the browser or
                         // applet viewer.
      invalid = false;
    } // end if

    g.drawImage(offScreenBuffer,0,0,this);
               // And now we transfer the info in the buffer onto the
               // graphics context we got from the browser in one smooth
               // motion.
  } // end paint

} // end DBCanvas

/* ******************************************************************
   end of file
*********************************************************************/
