/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : JavaNotes
   date       : 31.10.97
   description: the java notes client, which can run as applet, as
                application or as standalone application

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class JavaNotes extends Applet
  implements Runnable {

  // class variable declarations
  final static String USAGE    = "usage: java JavaNotes [-s] [-v:int]"+
                                 "[-h:Host] [-p:Port]"+
                                 "[-f:Pwdfile] [-d:Datadir]\n"+
                                 "       host and port must be spezified "+
                                 "except in standalone mode (-s)\n"+
                                 "       the size of the scroll panes is "+
                                 "adjustable by using the -v parameter\n"+
                                 "       -e:cmdline can execute a helper "+
                                 "application for displaying the help index";
  final static String DUP_PAR  = "Duplicate parameter";
  final static String INV_FILE = " is not an existing readable file";
  final static String INV_DIR  = " is not an existing readable directory";
  final static String INV_NUM  = "Invalid number format in ";
  final static String NO_PORT  = "Port parameter not specified in HTML page";
  final static String DEF_PWD  = "passwd";
  final static String DEF_DIR  = ".";
  final static int DEF_VSIZE   = 350;  // vertical size of the scroll panes
  final static int MIN_VSIZE   = 100;
  final static int MAX_VSIZE   = 800;
  final static int DIFF_VSIZE  = 120;  // diff. betw. applet & scrollpane size
  final static String DOC_TYPE = "JavaNotes Server";
  final static String DOC_REL  = "1.0";
  final static String COPY_R1  = "JavaNotes 1.0.1";
  final static String COPY_R2  = "(C) by Puehringer Edgar 1998";
  final static String COPY_R3  = "(C) by Phringer Edgar 1998";

  private final static String FRAME_TITLE = "Java Notes";
  private final static int UPDATE_SPEED   = 10000; // milliseconds
  private static boolean inApplet         = true;
  private static boolean inStandalone     = false;
  private static boolean initFail         = false;
  private static String pwdFilename       = null;
  private static String dataDir           = null;
  private static String host              = null;
  private static int port                 = -1;
  private static int vSize                = -1;
  private static String helpDoc           = null;
  private static String args[]            = null;
  private static Frame frame              = null;
  private static ConsoleProgressBar progress;

  // instance variable declarations
  private JavaNotesGUI gui;
  private CalendarDatabase cDatabase;
  private Thread thread;
  private boolean makeUpdate;

  // inner classes
  class QuitListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      e.getWindow().dispose();
    } // end windowClosing

    public void windowClosed(WindowEvent e) {
      stop();
      destroy();
      System.exit(0);
    } // end windowClosed

  } // end QuitListener

  // constructors
  public JavaNotes () {
    super();
  } // end constructor JavaNotes

  // methods
  public static void main(String cmdArgs[]) {
    System.out.print(COPY_R1+" "+COPY_R2+"\n");
    progress = new ConsoleProgressBar("Loading ",8);
    progress.inc(1);
    inApplet            = false;
    args                = cmdArgs;
    JavaNotes javaNotes = new JavaNotes();
    progress.inc(1);
    frame = new Frame(FRAME_TITLE);
    frame.setSize(300,200);
    frame.add(javaNotes);
    javaNotes.init();
    progress.inc(2);
    frame.pack();
    progress.inc(1);
    frame.show();
    progress.inc(2);
    javaNotes.start();
    progress.inc(1);
    progress.dispose();
  } // end main

  private void readArgs() {
    int i;

    if (inApplet) {
      helpDoc = getParameter("helpindex");
      String str = getParameter("port");
      if (str == null) {
        this.add(new Label("Error: "+NO_PORT));
        initFail    = true;
      }
      else {
        try {
          port = Integer.parseInt(str);
        } catch (NumberFormatException excep) { port = -1; }
        if (port < 0) {
          this.add(new Label("Error: "+INV_NUM+"port parameter"));
          initFail    = true;
        } // end if
      } // end if
      host  = getCodeBase().getHost();
      vSize = getSize().height-DIFF_VSIZE;
      if (vSize < MIN_VSIZE) vSize = MIN_VSIZE;

    }
    else {
      for (i=0; i<args.length; i++) {
        if (args[i].equals("-s")) {
          if (inStandalone) fail(null,DUP_PAR);
          inStandalone = true;
        }
        else if (args[i].startsWith("-h:")) {
          if (host != null) fail(null,DUP_PAR);
          host = args[i].substring(3);
        }
        else if (args[i].startsWith("-e:")) {
          if (helpDoc != null) fail(null,DUP_PAR);
          helpDoc = args[i].substring(3);
        }
        else if (args[i].startsWith("-p:")) {
          if (port >= 0) fail(null,DUP_PAR);
          try {
            port = Integer.parseInt(args[i].substring(3));
          } catch (NumberFormatException excep) { port = -1; }
          if (port < 0) fail(null,INV_NUM+'"'+args[i]+'"');
        }
        else if (args[i].startsWith("-f:")) {
          if (pwdFilename != null) fail(null,DUP_PAR);
          pwdFilename = args[i].substring(3);
        }
        else if (args[i].startsWith("-d:")) {
          if (dataDir != null) fail(null,DUP_PAR);
          dataDir = args[i].substring(3);
        }
        else if (args[i].startsWith("-v:")) {
          if (vSize >= 0) fail(null,DUP_PAR);
          try {
            vSize = Integer.parseInt(args[i].substring(3));
          } catch (NumberFormatException excep) { vSize = -1; }
          if (vSize < 0) fail(null,INV_NUM+'"'+args[i]+'"');
          if (vSize < MIN_VSIZE) vSize = MIN_VSIZE;
          if (vSize > MAX_VSIZE) vSize = MAX_VSIZE;
        }
        else {
          System.out.println(USAGE);
          System.exit(0);
        } // end if
      } // end for
      // some parameters must be spezified, except in standalone mode
      if ((!inStandalone) && ((host == null) || (port < 0))) {
        System.out.println(USAGE);
        System.exit(0);
      } // end if
      // if some parameters are not spezified, use default values
      if (pwdFilename == null) pwdFilename = DEF_PWD;
      if (dataDir == null)     dataDir     = DEF_DIR;
      if (vSize < 0)           vSize       = DEF_VSIZE;
    } // end if
  } // end readArgs

  public void init() {
    readArgs();
    if (inStandalone) {
      File pwdFile = new File(pwdFilename);
      if (!(pwdFile.exists() && pwdFile.isFile() && pwdFile.canRead()))
        fail(null,'"'+pwdFilename+'"'+INV_FILE);
      // end if
      File dirFile = new File(dataDir);
      if (!(dirFile.exists() && dirFile.isDirectory() && dirFile.canRead()))
        fail(null,'"'+dataDir+'"'+INV_DIR);
      // end if
      try {
        PwdTable pwdTable = new PwdTable(pwdFile);
        FilePool filePool = new FilePool(new DocVer(DOC_TYPE,DOC_REL),
                                         dirFile);
        cDatabase = new FileDatabase(pwdTable,filePool);
      } catch (IOException excep) {
        fail(excep,"Could not load user information");
      }
    }
    else {
      try {
        cDatabase = new WebDatabase(new Socket(host,port));
        //cDatabase = new DummyDatabase(null);
      } catch (IOException excep) {
        fail(excep,"Could not connect to JavaNotesServer");
      }
    } // end if
    if (frame == null) frame = new Frame(FRAME_TITLE);
    if (!inApplet) {
      frame.setIconImage(createImage(new MemoryImageSource(DukeImagePixmap.WIDTH,
         DukeImagePixmap.HEIGHT,DukeImagePixmap.PIXELS,0,DukeImagePixmap.WIDTH)));
      frame.addWindowListener(new QuitListener());
    } // end if
    gui = new JavaNotesGUI(cDatabase,frame,inApplet,vSize,helpDoc);
    setLayout(new GridLayout(1,1));
    add(gui);
    thread     = new Thread(this);
    makeUpdate = false;
    thread.start();
  } // end init

  public String getAppletInfo() {
    return COPY_R1+"\n"+COPY_R3;
  } // end getAppletInfo

  public String [][] getParameterInfo() {
    String pinfo[][] = {
      {"port",">0","port number of the JavaNotes Server"},
      {"helpindex","existing html file",
       "the index of the JavaNotes Help System"}
    };
    return pinfo;
  } // end getParameterInfo

  public void start() {
    makeUpdate = true;
  } // end start

  public void stop() {
    makeUpdate = false;
    //if (inApplet) {
    //  AudioClip audioClip = getAudioClip(getDocumentBase(),"Terminat.au");
    //  audioClip.play();
    //} // end if
  } // end stop

  public void run() {
    while (true) {
      try {
        Thread.currentThread().sleep(UPDATE_SPEED);
      } catch (InterruptedException e) {}
      if (makeUpdate) gui.updateNow();
    } // end while
  } // end run

  public void destroy() {
    String errorMsg;

    cDatabase.logout();
    errorMsg = cDatabase.getErrorMsg();
    if (errorMsg != null)
      System.err.println("Error on logout:\n"+errorMsg);
    cDatabase.endSession();
    errorMsg = cDatabase.getErrorMsg();
    if (errorMsg != null)
      System.err.println("Error on end of session:\n"+errorMsg);
    Assert.out.println("JavaNotes: destroy executed");
  } // end destroy

  private static void fail(Exception excep, String msg) {
    if (excep == null)
      System.err.println(msg);
    else
      System.err.println(msg+"."+excep);
    // end if
    System.exit(1);
  } // end fail

} // end JavaNotes

/* ******************************************************************
   end of file
*********************************************************************/
