/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.3)

   class      : PwdTable
   date       : 4.10.97
   description: colects user accounts and passwords in a hashtable,
                which is loaded from a file.

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

//import java.applet.*;
//import java.awt.*;
//import java.awt.image.*;
//import java.awt.peer.*;
import java.io.*;
//import java.lang.*;
//import java.net.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class PwdTable extends Hashtable {
  // instance variable declarations
  File pwdFile;

  // constructors
  public PwdTable(File passwordFile) throws IOException {
    super();

    FileInputStream inStream  = new FileInputStream(passwordFile);
    StreamTokenizer tokStream = new StreamTokenizer(new InputStreamReader(inStream));
    tokStream.wordChars('*','*');
    int curTok                = tokStream.nextToken();
    int wordCnt               = 0;
    int lineNo                = tokStream.lineno();
    PwdEntry pwdEntry         = new PwdEntry();
    String userName           = new String();

    pwdFile = passwordFile;
    while (curTok != StreamTokenizer.TT_EOF) {
      if (lineNo != tokStream.lineno()) {
        wordCnt = 0;
        lineNo = tokStream.lineno();
      } // end if
      if ((curTok == StreamTokenizer.TT_WORD) ||
          (curTok == '"') || (curTok == '\u0027')) {
        switch (wordCnt) {
          case 0: userName = tokStream.sval; break;
          case 1: pwdEntry.fileNameShortcut = tokStream.sval; break;
          case 2: if (tokStream.sval.equals("*"))
                    pwdEntry.password = "";
                  else  
                    pwdEntry.password = tokStream.sval;
                  // end if
                  put(userName,pwdEntry);
                  pwdEntry = new PwdEntry();
        } // end switch
        wordCnt++;
      } // end if
      curTok = tokStream.nextToken();
    } // end while
    inStream.close();
  } // end constructor pwdTable

  // methods
  String getFileNameShortcut(Object key) {
    PwdEntry pwdEntry = (PwdEntry)get(key);

    return pwdEntry.fileNameShortcut;
  } // end getFileNameShortcut

  String getPassword(Object key) {
    PwdEntry pwdEntry = (PwdEntry)get(key);

    return pwdEntry.password;
  } // end getPassword

  void changePassword(Object key, String password) throws IOException {
    if (!containsKey(key)) return;
    PwdEntry userData = (PwdEntry)get(key);
    File bakFile = new File(pwdFile.getAbsolutePath()+".bak");
    if (bakFile.exists()) bakFile.delete();
    bakFile = new File(pwdFile.getAbsolutePath());
    bakFile.renameTo(new File(bakFile.getAbsolutePath()+".bak"));
    bakFile = new File(bakFile.getAbsolutePath()+".bak");
    FileInputStream inStream   = new FileInputStream(bakFile);
    FileOutputStream outStream = new FileOutputStream(pwdFile);
    BufferedReader dataIn      = new BufferedReader(new InputStreamReader(inStream));
    PrintWriter printOut       = new PrintWriter(outStream);
    String line                = dataIn.readLine();
    while (line != null) {             // while not end of file
      if (containsUser(line,key,userData)) {
        // write a modified line
        printOut.print('"'); printOut.print(key); printOut.print('"');
        printOut.print(':');
        printOut.print(userData.fileNameShortcut);
        printOut.print(':');
        if (password.equals(""))
          printOut.println("*");
        else
          printOut.println(password);
        // end if
        userData.password = password;
        put(key,userData);
      }
      else { // write original line
        printOut.println(line);
      } // end if
      line = dataIn.readLine();
    } // end while
    inStream.close();
    printOut.close();
    outStream.close();
  } // end changePassword

//  void addUser(Object key, String pwd, String shortCut) throws IOException {
//    if (containsKey(key)) return;
//    RandomAccessFile raf = new RandomAccessFile(pwdFile,"rw");
//    char ch;
//    PwdEntry userData = new PwdEntry(shortCut, pwd);

//    raf.seek(raf.length()-1);
//    ch = raf.readChar();
//    if (ch != '\n') raf.writeChar('\n');
//    raf.writeChar('"');
//    raf.writeUTF(key);
//    raf.writeChar('"');
//    raf.writeChar(':');
//    raf.writeUTF(userData.fileNameShortcut);
//    raf.writeChar(':');
//    if (pwd.equals(""))
//      raf.writeUTF("*");
//    else
//      raf.writeUTF(pwd);
    // end if
//    raf.close();
//  } // end addUser

  private boolean containsUser(String str, Object userName, PwdEntry userData) {
    String password = userData.password.equals("") ? "*" : userData.password;

    if ((str.indexOf(userName.toString()) >= 0) &&
        (str.indexOf(userData.fileNameShortcut) >= 0) &&
        (str.indexOf(password) >= 0)) {
      return true;
    } // end if
    return false;
  } // end containsUser

} // end PwdTable

/* ******************************************************************
   class declaration
*********************************************************************/

class PwdEntry {

  // instance variable declarations
  String fileNameShortcut;
  String password;

  // constructors
  PwdEntry() {

  } // end constructor PwdEntry

  PwdEntry(String fileNameShortcut, String password) {

    this.fileNameShortcut = fileNameShortcut;
    this.password         = password;
  } // end constructor PwdEntry

} // end PwdEntry

/* ******************************************************************
   end of file
*********************************************************************/
