/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.3)

   class      : RequestHeader
   date       : 15.11.97
   description: a header for requests to the java notes server

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;
import java.util.*;

/* ******************************************************************
   interface
*********************************************************************/

public class RequestHeader implements Streamable {

  // class variable declarations
  public final static int LOGIN        = 1;
  public final static int LOGOUT       = 2;
  public final static int CHANGEPASSWD = 3;
  public final static int GETUSERS     = 4;
  public final static int GETDATES     = 5;
  public final static int GETMESSAGES  = 6;
  public final static int PUTDATE      = 7;
  public final static int DELETEDATE   = 8;
  public final static int UPDATEDATE   = 9;
  public final static int ENDSESSION   = 10;

  // instance variable declarations
  private int requestType    = -1;
  private String user        = "";     // for login and getDates request
  private String passwd      = "";     // for login and changePasswd request
  private Date from          = null;   // for getDates request
  private Date to            = null;   // for getDates request
  private boolean periodical = false;  // for getDates request
  private boolean forAll     = false;  // for deleteDate and updateDate request
  private boolean lastInSeq  = true;   // true if element is last in a
                                       // sequence

  // constructors
  public RequestHeader (int requestType, String user, String passwd) {
    // used for login requests
    this.requestType = requestType;
    this.user        = user;
    this.passwd      = passwd;
  } // end constructor RequestHeader

  public RequestHeader (int requestType) {
    // used for logout, getUsers, getMessages and putDate requests
    this.requestType = requestType;
    lastInSeq = !(requestType == PUTDATE);
  } // end constructor RequestHeader

  public RequestHeader (int requestType, String newPasswd) {
    // used for changePasswd requests
    this.requestType = requestType;
    passwd           = newPasswd;
  } // end constructor RequestHeader

  public RequestHeader (int requestType, String user, Date from, Date to,
                        boolean periodical) {
    // used for getDates requests
    this.requestType = requestType;
    this.user        = user;
    this.from        = from;
    this.to          = to;
    this.periodical  = periodical;
  } // end constructor RequestHeader

  public RequestHeader (int requestType, boolean forAll) {
    // used for deleteDate and updateDate requests
    this.requestType = requestType;
    this.forAll      = forAll;
    lastInSeq        = false;
  } // end constructor RequestHeader

  public RequestHeader (DataInput in) throws IOException {
    // used to create a RequestHeader instance from a stream

    inputFrom(in);
  } // end constructor RequestHeader

  // methods
  public void outputTo(DataOutput out) throws IOException {

    out.writeInt(requestType);
    out.writeUTF(user);
    out.writeUTF(passwd);
    if (from == null) out.writeLong(0);
      else out.writeLong(from.getTime());
    if (to == null) out.writeLong(0);
      else out.writeLong(to.getTime());
    out.writeBoolean(periodical);
    out.writeBoolean(forAll);
    out.writeBoolean(lastInSeq);
  } // end outputTo

  public void inputFrom(DataInput in) throws IOException {

    requestType = in.readInt();
    user        = in.readUTF();
    passwd      = in.readUTF();
    from        = new Date(in.readLong());
    to          = new Date(in.readLong());
    periodical  = in.readBoolean();
    forAll      = in.readBoolean();
    lastInSeq   = in.readBoolean();
  } // end inputFrom

  public void setLastInSeq(boolean lastInSeq) {
  // sets if object is the last in a sequence or not

    this.lastInSeq = lastInSeq;
  } // end setLastInSeq

  public boolean isLastInSeq() {
  // indicates if object is the last in a sequence or not

    return lastInSeq;
  } // end isLastInSeq

  public int getRequestType() {

    return requestType;
  } // end getRequestType

  public String getUser() {

    return user;
  } // end getUser

  public String getPasswd() {

    return passwd;
  } // end getPasswd

  public Date getFrom() {

    return from;
  } // end getFrom

  public Date getTo() {

    return to;
  } // end getTo

  public boolean isPeriodical() {

    return periodical;
  } // end isPeriodical

  public boolean isForAll() {

    return forAll;
  } // end is forAll;

} // end RequestHeader

/* ******************************************************************
   end of file
*********************************************************************/
