/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : ToolPanel
   date       : 02.01.98
   description: a panel class that performs toolbar operations
   modified   : 19.02.98 - auto insertion of start and end on new
                           operation added
                13.03.98 - most of the dialogs are set to visible
                           manually due to a bug while running
                           JavaNotes at Solaris

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class ToolPanel extends Panel
  implements Observer, ActionListener, ItemListener {

  // class variable declarations

  // instance variable declarations
  private Frame parent;
  private boolean inApplet;
  private Calendar calendar;
  private LoginInfoModel lModel;
  private ScopeModel sModel;
  private DateListModel dModel;
  private CalendarDatabase db;
  private Label userLabel;
  private Choice userChoice;
  private Button todayBtn;
  private Button goToBtn;
  private Button newBtn;
  private Button viewBtn;
  private Button delBtn;
  private LoginModelListener loginListener;

  // inner classes
  class DateDlgListener extends WindowAdapter {

    private DateChooserDlg dateDlg;

    public DateDlgListener(DateChooserDlg dateDlg) {
      super();
      this.dateDlg = dateDlg;
    } // end constructor DateDlgListener

    public void windowClosed(WindowEvent e) {
      if (!dateDlg.getCanceled()) {
        sModel.setDate(dateDlg.getDate());
      } // end if
    } // end windowClosing
  
  } // end DateDlgListener

  class EditDlgListener extends WindowAdapter {

    private EditDateDlg editDlg;
    private int mode;

    public EditDlgListener(EditDateDlg editDlg, int mode) {
      super();
      this.editDlg = editDlg;
      this.mode    = mode;
    } // end constructor EditDlgListener

    public void windowClosed(WindowEvent e) {
      if (!editDlg.getCanceled()) {
        handleEditDlgResult(editDlg,mode);
      } // end if
    } // end windowClosed
  
  } // end EditDlgListener

  class InsertYesNoListener extends WindowAdapter {

    private MsgBoxDlg msgDlg;
    private CalendarEntry entry;

    public InsertYesNoListener(MsgBoxDlg msgDlg, CalendarEntry entry) {
      super();
      this.msgDlg = msgDlg;
      this.entry  = entry;
    } // end constructor InsertYesNoListener

    public void windowClosed(WindowEvent e) {
      if (msgDlg.getChoice() == MsgBoxDlg.IDYES) New(entry);
    } // end windowClosed
  
  } // end InsertYesNoListener

  class DeleteAllYesNoListener extends WindowAdapter {

    private MsgBoxDlg msgDlg;
    private CalendarEntry entry;

    public DeleteAllYesNoListener(MsgBoxDlg msgDlg, CalendarEntry entry) {
      super();
      this.msgDlg = msgDlg;
      this.entry  = entry;
    } // end constructor DeleteAllYesNoListener

    public void windowClosed(WindowEvent e) {
      switch (msgDlg.getChoice()) {
        case MsgBoxDlg.IDYES: internalDelete(entry,false); break;
        case MsgBoxDlg.IDNO:  internalDelete(entry,true);  break;
        default: // IDCANCEL
      } // end switch
    } // end windowClosed
  
  } // end DeleteAllYesNoListener

  class LoginModelListener implements Observer {

    public void update(Observable obs, Object arg) {
      updateLoginInfo();
    } // end update

    public void updateLoginInfo() {
      String[] personsList = lModel.getPersonsList();
      int i;
  
      userChoice.removeAll();
      for (i=0; i<personsList.length; i++) userChoice.addItem(personsList[i]);
      userChoice.select(lModel.getDefSelection());
      userLabel.setEnabled(lModel.getLogedIn());
      userChoice.setEnabled(lModel.getLogedIn());
      sModel.setUser(lModel.getPersons()[userChoice.getSelectedIndex()]);
      Assert.out.println("Toolpanel: login info updated");
    } // end updateLoginInfo

  } // end LoginModelListener

  // constructors
  public ToolPanel(Frame parent, boolean inApplet,
                   LoginInfoModel lModel, ScopeModel sModel,
                   DateListModel dModel, CalendarDatabase db) {
    this.parent   = parent;
    this.inApplet = inApplet;
    calendar      = CalendarCreator.get();
    this.lModel   = lModel;
    this.sModel   = sModel;
    this.dModel   = dModel;
    this.db       = db;
    loginListener = new LoginModelListener();
    //this.lModel.addObserver(this);
    this.lModel.addObserver(loginListener);
    this.sModel.addObserver(this);
    userLabel  = new Label("User");
    add(userLabel);
    userChoice = new Choice();
    userChoice.addItemListener(this);
    add(userChoice);
    Panel btnPanel = new Panel(new GridLayout(1,5,5,0));
    btnPanel.add(todayBtn = new Button("Today"));
    todayBtn.addActionListener(this);
    btnPanel.add(goToBtn  = new Button("Go To"));
    goToBtn.addActionListener(this);
    btnPanel.add(newBtn  = new Button("New"));
    newBtn.addActionListener(this);
    btnPanel.add(viewBtn  = new Button("View"));
    viewBtn.addActionListener(this);
    btnPanel.add(delBtn  = new Button("Delete"));
    delBtn.addActionListener(this);
    add(btnPanel);
    loginListener.updateLoginInfo();
    updateScope();
  } // end constructor ToolPanel

  // methods
  public void update(Observable obs, Object arg) {
    // this is called if the observable changes
    //if (obs instanceof LoginInfoModel)
    //  updateLoginInfo();
    //else
      updateScope();
    // end if
  } // end update

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Today")) {
      sModel.setDate(new Date());
    }
    else if (cmd.equals("Go To")) {
      calendar.setTime(sModel.getDate());
      DateChooserDlg dateDlg = new DateChooserDlg(parent,"Select Date",
                                                  false,calendar,
                                                  CalendarCreator.getLocale());
      dateDlg.addWindowListener(new DateDlgListener(dateDlg));
      dateDlg.setVisible(true);
    }
    else if (cmd.equals("New")) {
      CalendarEntry entry = new CalendarEntry(lModel.getUser());
      Date start          = sModel.getSelectedStart();
      if (start != null) {
        Date end = sModel.getSelectedEnd();
        entry.setStart(start);
        entry.setEnd(end);
      } // end if
      EditDateDlg editDlg = new EditDateDlg(parent,"New Date",!inApplet,
                                            entry,EditDateDlg.NEW,
                                            lModel.getPersons());
      if (inApplet) editDlg.setAutoCancel(true);
      editDlg.addWindowListener(new EditDlgListener(editDlg,EditDateDlg.NEW));
      editDlg.setVisible(true);
    }
    else if (cmd.equals("View")) {
      CalendarEntry entry = sModel.getSelectedObject();
      int mode            = delBtn.isEnabled() ? EditDateDlg.VIEW_AND_DELETE
                                               : EditDateDlg.VIEW;
      EditDateDlg editDlg = new EditDateDlg(parent,"View Date",!inApplet,
                                            entry,mode,
                                            lModel.getPersons());
      if (inApplet) editDlg.setAutoCancel(true);
      editDlg.addWindowListener(new EditDlgListener(editDlg,mode));
      editDlg.setVisible(true);
    }
    else if (cmd.equals("Delete")) {
      Delete((CalendarEntry)sModel.getSelectedObject());
    }
    else {
      System.out.println(e.getActionCommand());
    } // end if
  } // end actionPerformed

  public void handleEditDlgResult(EditDateDlg editDlg, int mode) {
    if ((mode == EditDateDlg.NEW) ||
        (mode == EditDateDlg.FIND_RESULT_NEW)) {
      CalendarEntry entry = editDlg.getCalendarEntry();
      if (dModel.isOverlapping(entry)) {
        MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,"New",
                                    !inApplet,
                                    "The new Date overlaps with an "+
                                    "existing Date.\n"+
                                    "Insert anyway?",
                                    MsgBoxDlg.YESNO |
                                    MsgBoxDlg.ICONQUESTION |
                                    MsgBoxDlg.DEFBUTTON2);
        if (inApplet) msgBoxDlg.setAutoCancel(true);
        msgBoxDlg.addWindowListener(new InsertYesNoListener(msgBoxDlg,
                                          entry));
        msgBoxDlg.setVisible(true);
      }
      else {
        New(entry);
      } // end if
    }
    else if ((mode == EditDateDlg.VIEW_AND_DELETE) &&
             (editDlg.getDeleted())) {
      Delete(editDlg.getCalendarEntry());
    } // end if
  } // end handleEditDlgResult

  public void Delete(CalendarEntry entry) {
    if (entry.isPeriodic()) {
      MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,"Delete",
                                  !inApplet,
                                  "Delete all occurrences of this"+
                                  " periodic date?",
                                  MsgBoxDlg.YESNOCANCEL |
                                  MsgBoxDlg.ICONQUESTION |
                                  MsgBoxDlg.DEFBUTTON2);
      if (inApplet) msgBoxDlg.setAutoCancel(true);
      msgBoxDlg.addWindowListener(new DeleteAllYesNoListener(msgBoxDlg,
                                        entry));
      msgBoxDlg.setVisible(true);
    }
    else {
      internalDelete(entry,false);
    } // end if
  } // end Delete

  private void internalDelete(CalendarEntry entry, boolean onlyThisInstance) {
    // assert onlyThisInstance is false, if date is not periodic
    String errorMsg;
    CalendarEntry toDelete;
    boolean deleteForAll = sModel.getUser().equals(entry.getCreator());

    if (onlyThisInstance) {
      toDelete = (CalendarEntry)entry.clone();
      toDelete.setPeriodic(CalendarEntry.DELETED_INSTANCE);
      if (!deleteForAll) {
        toDelete.setPersons(new String[0]);
        toDelete.setCreator(sModel.getUser());
      } // end if
      db.putDate(toDelete);
    }
    else {
      toDelete = entry.reconstructParent();
      db.deleteDate(toDelete,deleteForAll);
    } // end if
    errorMsg = db.getErrorMsg();
    if (errorMsg != null) {
      (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
    }
    else {
      if (toDelete.isPeriodic())
        dModel.periodicInvalidate();
      else if (sModel.matchesScope(toDelete.getStart()))
        dModel.invalidate();
      // end if
    } // end if
  } // end internalDelete

  public void New(CalendarEntry entry) {
    String errorMsg;

    db.putDate(entry);
    errorMsg = db.getErrorMsg();
    if (errorMsg != null) {
      (new ErrorDlg(parent,"Error",!inApplet,errorMsg)).setAutoCancel(inApplet);
    }
    else {
      if (entry.isPeriodic())
        dModel.periodicInvalidate();
      else if (sModel.matchesScope(entry.getStart()))
        dModel.invalidate();
      // end if
    } // end if
  } // end New

  public void itemStateChanged(ItemEvent e) {
    if (lModel.getLogedIn()) {
      sModel.setUser(lModel.getPersons()[userChoice.getSelectedIndex()]);
    }
    else {
      sModel.setUser(null);
    } // end if
  } // end itemStateChanged

//  private void updateLoginInfo() {
//    String[] personsList = lModel.getPersonsList();
//    int i;

//    userChoice.removeAll();
//    for (i=0; i<personsList.length; i++) userChoice.addItem(personsList[i]);
//    userChoice.select(lModel.getDefSelection());
//    userLabel.setEnabled(lModel.getLogedIn());
//    userChoice.setEnabled(lModel.getLogedIn());
//    sModel.setUser(lModel.getPersons()[userChoice.getSelectedIndex()]);
//  } // end updateLoginInfo

  private void updateScope() {
    boolean selection   = (sModel.getSelectedObject() != null);
    boolean changeRight = lModel.getUser().equals(sModel.getUser());

    newBtn.setEnabled(!selection && changeRight);
    viewBtn.setEnabled(selection);
    delBtn.setEnabled(selection && changeRight);
    Assert.out.println("Toolpanel: scope updated");
  } // end updateScope

} // end ToolPanel

/* ******************************************************************
   end of file
*********************************************************************/
